"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasCleanIndex = exports.getGitFileList = exports.getDiffFileList = exports.getRangesForDiff = exports.getDiffForFile = void 0;
const child_process = __importStar(require("child_process"));
const path = __importStar(require("path"));
const Range_1 = require("./Range");
const sanitizeFilePath = (filePath) => JSON.stringify(path.resolve(filePath));
const diffCacheKey = (filePath, staged) => JSON.stringify([path.resolve(filePath), staged]);
const setCachedDiff = (filePath, staged, diff) => void diffCache.set(diffCacheKey(filePath, staged), diff);
const getCachedDiff = (filePath, staged) => diffCache.get(diffCacheKey(filePath, staged));
const diffCache = new Map();
const getDiffForFile = (filePath, staged = false) => {
    var _a;
    let diff = getCachedDiff(filePath, staged);
    if (diff === undefined) {
        const command = [
            "git",
            "diff",
            "--diff-filter=ACM",
            "--relative",
            staged && "--staged",
            "--unified=0",
            JSON.stringify((_a = process.env.ESLINT_PLUGIN_DIFF_COMMIT) !== null && _a !== void 0 ? _a : "HEAD"),
            "--",
            sanitizeFilePath(filePath),
        ]
            .filter(Boolean)
            .join(" ");
        const result = child_process.execSync(command).toString();
        setCachedDiff(filePath, staged, result);
        diff = result;
    }
    return diff;
};
exports.getDiffForFile = getDiffForFile;
let diffFileListCache;
const getDiffFileList = (staged = false) => {
    var _a;
    if (diffFileListCache === undefined) {
        const command = [
            "git",
            "diff",
            "--diff-filter=ACM",
            "--name-only",
            "--relative",
            staged && "--staged",
            JSON.stringify((_a = process.env.ESLINT_PLUGIN_DIFF_COMMIT) !== null && _a !== void 0 ? _a : "HEAD"),
        ]
            .filter(Boolean)
            .join(" ");
        diffFileListCache = child_process
            .execSync(command)
            .toString()
            .trim()
            .split("\n")
            .map((filePath) => path.resolve(filePath));
    }
    return diffFileListCache;
};
exports.getDiffFileList = getDiffFileList;
let gitFileListCache;
const getGitFileList = () => {
    if (gitFileListCache === undefined) {
        const command = ["git", "ls-files"].filter(Boolean).join(" ");
        gitFileListCache = child_process
            .execSync(command)
            .toString()
            .trim()
            .split("\n")
            .map((filePath) => path.resolve(filePath));
    }
    return gitFileListCache;
};
exports.getGitFileList = getGitFileList;
const hasCleanIndex = (filePath) => {
    const command = [
        "git",
        "diff",
        "--quiet",
        "--relative",
        "--unified=0",
        "--",
        sanitizeFilePath(filePath),
    ].join(" ");
    let result = true;
    try {
        child_process.execSync(command).toString();
    }
    catch (err) {
        result = false;
    }
    return result;
};
exports.hasCleanIndex = hasCleanIndex;
const isHunkHeader = (input) => {
    const hunkHeaderRE = new RegExp(/^@@ [^@]* @@/);
    return hunkHeaderRE.exec(input);
};
const getRangeForChangedLines = (line) => {
    /**
     * Example values of the RegExp's group:
     *
     * start: '7',
     * linesCountDelimiter: ',2',
     * linesCount: '2',
     */
    const rangeRE = new RegExp(/^@@ .* \+(?<start>\d+)(?<linesCountDelimiter>,(?<linesCount>\d+))? @@/);
    const range = rangeRE.exec(line);
    if (range === null) {
        throw Error(`Couldn't match regex with line '${line}'`);
    }
    const groups = {
        // Fallback value to ensure hasAddedLines resolves to false
        start: "0",
        linesCountDelimiter: ",0",
        linesCount: "0",
        ...range.groups,
    };
    const linesCount = groups.linesCountDelimiter && groups.linesCount
        ? parseInt(groups.linesCount)
        : 1;
    const hasAddedLines = linesCount !== 0;
    const start = parseInt(groups.start);
    const end = start + linesCount;
    return hasAddedLines ? new Range_1.Range(start, end) : null;
};
const getRangesForDiff = (diff) => diff.split("\n").reduce((acc, cur) => {
    if (!isHunkHeader(cur)) {
        return acc;
    }
    const range = getRangeForChangedLines(cur);
    if (range === null) {
        return acc;
    }
    return [...acc, range];
}, []);
exports.getRangesForDiff = getRangesForDiff;
